package com.agentfactory.agentspeak.compiler;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.agentfactory.agentspeak.interpreter.Utilities;
import com.agentfactory.agentspeak.interpreter.mentalstate.ASEvent;
import com.agentfactory.agentspeak.interpreter.mentalstate.IPlanStep;
import com.agentfactory.agentspeak.interpreter.mentalstate.PlanRule;
import com.agentfactory.logic.update.lang.AND;
import com.agentfactory.logic.update.lang.Predicate;

public class CodeGeneratorVisitor implements AgentSpeakParserVisitor {
	public Map<Predicate, String> actions = new HashMap<Predicate, String>();
	public Map<Predicate, String> sensors = new HashMap<Predicate, String>();
	public Map<Predicate, String> modules = new HashMap<Predicate, String>();
	public List<PlanRule> rules = new LinkedList<PlanRule>();
	public List<Predicate> beliefs = new LinkedList<Predicate>();

	private static int counter = 0;
	
	@SuppressWarnings("unchecked")
	@Override
	public Object visit(SimpleNode node, Object data) {
		PredicateBuilderVisitor visitor = new PredicateBuilderVisitor();
		if (node.toString().equals("SensorExpression")) {
			String predicate = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			String cname = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
			if (cname != null) {
				sensors.put((Predicate) Utilities.factory.createFormula(predicate), cname);
			}
		} else if (node.toString().equals("ActionExpression")) {
			String predicate = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			String cname = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
			if (cname != null) {
				actions.put((Predicate) Utilities.factory.createFormula(predicate), cname);
			}
		} else if (node.toString().equals("ModuleExpression")) {
			String predicate = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			String cname = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
			if (cname != null) {
				modules.put((Predicate) Utilities.factory.createFormula(predicate), cname);
			}
		} else if (node.toString().equals("RuleExpression")) {
			int i = 0;
			String label = null;
			if (node.jjtGetNumChildren() == 4) {
				label = (String) new PredicateBuilderVisitor().visit((SimpleNode) node.jjtGetChild(i++), null);
			} else {
				label = "rule" + counter++;
			}
			
			SimpleNode eventNode = (SimpleNode) node.jjtGetChild(i++);
			int type = (eventNode.jjtGetValue().equals("+")) ? ASEvent.ADDITION : ASEvent.REMOVAL;
			String predicateEvent = (String) visitor.visit((SimpleNode) eventNode.jjtGetChild(0), null);
			
			AND and = (AND) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(i++), null);
			List<IPlanStep> plan = (List<IPlanStep>) new PlanBuilderVisitor().visit((SimpleNode) node.jjtGetChild(i++), null);
			
			rules.add(new PlanRule( (Predicate) Utilities.factory.createFormula(label),
									new ASEvent(type, (Predicate) Utilities.factory.createFormula(predicateEvent)),
									and, plan));
		} else if (node.toString().equals("Predicate")) {
			String predicate = (String) new PredicateBuilderVisitor().visit(node, null);
			beliefs.add((Predicate) Utilities.factory.createFormula(predicate));
		} else if (node.toString().equals("IncludeExpression")) {
			// IGNORE: This should never be called
		} else {
			for (int i=0; i<node.jjtGetNumChildren(); i++) {
				visit((SimpleNode) node.jjtGetChild(i), data);
			}
		}
		return null;
	}

}
