package com.agentfactory.agentspeak.interpreter.control;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.agentfactory.agentspeak.interpreter.Utilities;
import com.agentfactory.agentspeak.interpreter.mentalstate.ASEvent;
import com.agentfactory.agentspeak.interpreter.mentalstate.PlanRule;

/**
 * This class implements the event selection policy for the interpreter. The
 * basic implementation involves an eventFilter that is a set of all triggering
 * events declared in the agents plan rules.
 * 
 * The selector method (next) returns the first event from the event queue that
 * has been registered with the eventFilter. If there is no event, then the method
 * returns null.
 * 
 * @author Rem Collier
 *
 */
public class BasicEventSelector implements IEventSelector {
	private List<ASEvent> eventQueue;
	private Set<String> eventFilter;
	
	public BasicEventSelector(List<ASEvent> eventQueue) {
		this.eventQueue = eventQueue;
		eventFilter = new HashSet<String>();
	}
	
	/**
	 * Core event selector method
	 * 
	 * @return the next relevant event or null
	 */
	@Override
	public ASEvent next() {
		if (eventQueue.isEmpty()) return null;
		
		ASEvent event = null;
		String sig = null;
		do {
			event = eventQueue.remove(0);
			sig = Utilities.createSignature(event.content());
		} while (!eventQueue.isEmpty() && !eventFilter.contains(sig));
		return eventFilter.contains(sig) ? event:null;
	}

	/**
	 * Add the triggering event of the plan rule to the event filter.
	 */
	@Override
	public void registerPlanRule(PlanRule rule) {
		eventFilter.add(Utilities.createSignature(rule.getTriggeringEvent().content()));
	}

}
