package com.agentfactory.agentspeak.interpreter.mentalstate;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.agentspeak.interpreter.AbstractAgentSpeakAgent;
import com.agentfactory.agentspeak.interpreter.Utilities;
import com.agentfactory.logic.update.lang.IFormula;
import com.agentfactory.logic.update.reasoner.Bindings;
import com.agentfactory.logic.update.reasoner.QueryResult;

public class ForEachStep implements IPlanStep {
	private IFormula guard;
	private List<IPlanStep> plan;
	
	public ForEachStep(IFormula guard, List<IPlanStep> plan) {
		this.guard = guard;
		this.plan = plan;
	}

	@Override
	public IPlanStep apply(Bindings set) {
		List<IPlanStep> list = new LinkedList<IPlanStep>();
		for (IPlanStep step: plan) {
			list.add(step.apply(set));
		}
		return new ForEachStep(guard.apply(set), list);
	}

	public String toString() {
		String output = "foreach(" + Utilities.presenter.toString(guard) + ") {";
		boolean first = true;
		for (IPlanStep step: plan) {
			if (first) first = false;
			else output += ",";
			output += step.toString();
		}
		return output + "}";
	}

	public IFormula guard() {
		return guard;
	}
	
	public List<IPlanStep> plan() {
		return plan;
	}

	@Override
	public void handle(AbstractAgentSpeakAgent agent, Commitment commitment) {
		commitment.pushBindings();
		QueryResult result = agent.queryAll(guard.apply(commitment.bindings()));
		if (result.found) {
			if (result.bindings.isEmpty()) {
				List<IPlanStep> steps = new LinkedList<IPlanStep>();
				for (IPlanStep stp: plan) {
					steps.add(stp);
				}
				commitment.addPlan(steps);
			} else {
				for (Bindings binding: result.bindings) {
					List<IPlanStep> steps = new LinkedList<IPlanStep>();
					for (IPlanStep stp: plan) {
						steps.add(stp.apply(binding));
					}
					commitment.addPlan(steps);
				}
			}
		}
		commitment.popBindings();
	}
}
