package com.agentfactory.logic.update.reasoner;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import com.agentfactory.logic.update.interpreter.CoreUtilities;
import com.agentfactory.logic.update.lang.ITerm;
import com.agentfactory.logic.update.lang.Variable;

public class Bindings implements Iterable<ITerm[]> {
	private List<ITerm[]> bindings;
	
	public Bindings() {
		bindings = new LinkedList<ITerm[]>();
	}
	
	public void addBinding(ITerm term, ITerm term2) {
		for (ITerm[] terms: bindings) {
			if (terms[0].equals(term)) {
				terms[1] = term2;
				return;
			}
		}
		bindings.add(new ITerm[] { term, term2 });
	}
	
	public ITerm getBinding(ITerm term) {
		for (ITerm[] terms: bindings) {
			if (terms[0].equals(term)) return terms[1];
			//if (terms[1].equals(term)) return terms[0];
		}
		return null;
	}
	
	public Bindings merge(Bindings bindingSet) {
		Bindings b = new Bindings();
		b.bindings.addAll(bindings);
		for (ITerm[] terms: bindingSet.bindings) {
			b.addBinding(terms[0], terms[1]);
		}
		return b;
	}

	@Override
	public Iterator<ITerm[]> iterator() {
		return bindings.iterator();
	}
	
    /**
     * Generate an Idempotent Binding
     * @return
     */
    public Bindings idempotencyBuilder() {
        Bindings subs = new Bindings();
        for (ITerm[] terms: bindings) {
        	if (!Variable.class.isInstance(terms[0])) {
        		terms[0] = terms[0].apply(this);
        	}
        	if (!Variable.class.isInstance(terms[1])) {
        		terms[1] = terms[1].apply(this);
        	}
        	subs.addBinding(terms[0], terms[1]);
        }
        return subs;
    }
    
    public boolean equals(Object object) {
    	if (Bindings.class.isInstance(object)) {
    		Bindings b = (Bindings) object;
    		for (ITerm[] terms: bindings) {
    			ITerm t = b.getBinding(terms[0]);
    			if (t == null) {
    				return false;
    			}
    			if (!terms[1].equals(t)) {
    				return false;
    			}
    		}
    		for (ITerm[] terms: b.bindings) {
    			if (getBinding(terms[0]) == null) {
    				return false;
    			}
    		}
    		return true;
    	}
    	return false;
    }

    public Bindings mostGeneralUnifier() {
    	Bindings subs = new Bindings();
        for (ITerm[] binding: bindings) {
            subs.addBinding(binding[0], binding[1].apply(this));
        }
        return subs;
    }
    
    public Bindings filter(Set<Variable> variables) {
    	Bindings b = new Bindings();
    	for (Variable variable: variables) {
    		ITerm term = getBinding(variable);
    		if (term != null) {
    			b.addBinding(variable, term);
    		}
    	}
    	return b;
    }
}
