/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.agentfactory.visualiser.impl;

import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.core.IMessage;
import com.agentfactory.platform.core.IPlatformService;
import com.agentfactory.visualiser.core.ISnapshot;
import com.agentfactory.visualiser.core.IStateManager;
import java.util.LinkedList;
import java.util.List;
import java.util.Observable;

/**
 *
 * @author remcollier
 */
public class DefaultStateManager extends Observable implements IStateManager {
    private static final int CAPACITY = 200;
    
    private long iteration;
    protected IAgent agent;
    private DefaultSnapshot snapshot;
    private List<DefaultSnapshot> history;
    
    public void setAgent(IAgent agent) {
        this.agent = agent;
        history = new LinkedList<DefaultSnapshot>();
        capture();
    }

    public void capture() {
        DefaultSnapshot shot = createSnapshot();
        constructSnapshot(shot);
        
        history.add(shot);
        if (history.size() > CAPACITY) {
            history.remove(0);
        }
        
        snapshot = shot;
        
        this.iteration = snapshot.numIterations();
        setChanged();
        notifyObservers(NEW_SNAPSHOT);
    }
    
    protected void constructSnapshot(DefaultSnapshot shot) {
        shot.name = agent.getName();
        shot.type = agent.getType();
        shot.iterations = agent.getIterations();
        shot.inbox = new LinkedList<IMessage>(agent.getInbox());
        shot.outbox = new LinkedList<IMessage>(agent.getOutbox());
        shot.services = new LinkedList<IPlatformService>(agent.getBoundPlatformServices());
    }
    
    protected DefaultSnapshot createSnapshot() {
        return new DefaultSnapshot();
    }
    
    public ISnapshot getCurrentSnapshot() {
        return snapshot;
    }
    
    public String getAgentName() {
        return agent.getName();
    }
    
    public String getAgentType() {
        return agent.getType();
    }
       
    public synchronized void first() {
        snapshot = history.get(0);
        iteration = 0;
        
        setChanged();
        notifyObservers(NEW_SNAPSHOT);
    }    
    
    public synchronized void prev() {
        if (iteration > history.get(0).numIterations()) {
            iteration--;
            snapshot = history.get((int) (iteration - history.get(0).numIterations()));
            
            setChanged();
            notifyObservers(NEW_SNAPSHOT);
        }
    }

    public synchronized void next() {
        if (iteration < history.get(history.size()-1).numIterations()) {
            iteration++;
            snapshot = history.get((int) (iteration - history.get(0).numIterations()));
            
            setChanged();
            notifyObservers(NEW_SNAPSHOT);
        }
    }
    
    public synchronized void last() {
        snapshot = history.get(history.size() - 1);
        iteration = snapshot.numIterations();
        
        setChanged();
        notifyObservers(NEW_SNAPSHOT);
    }
}
